#!/usr/bin/env python
#
#   import-index.py
#   zfilenet
#
#   Copyright (C) 2008 Ross Light
#
#   This file is part of zfilenet.
#
#   zfilenet is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   zfilenet is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with zfilenet.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Index server maintenence script

Reads the contents of an index interchange file (an XML format created from
export-index.py) from stdin to a new index database.

usage: import-index.py database
"""

from datetime import datetime
import os
import sys
from textwrap import dedent
from xml.etree import cElementTree as etree

import progtools

from zfilenet.db import DBVariable
from zfilenet.indexserver import index
from zfilenet.util import hex2data, to_uuid

__author__ = 'Ross Light'
__date__ = 'June 15, 2008'
__all__ = ['ZFN_INDEX_NAMESPACE',
           'INDEXOBJ_TAGS',
           'zfnxml',
           'decode_indexobj',
           'main',]

ZFN_INDEX_NAMESPACE = "http://launchpad.net/zfilenet/ns/index"

def zfnxml(tag):
    """
    Converts a tag to the ZFN index namespace.
    
    :Parameters:
        tag : str
            The name of the element
    :Returns: The qualified tag name
    :ReturnType: str
    """
    return '{%s}%s' % (ZFN_INDEX_NAMESPACE, tag)

INDEXOBJ_TAGS = {zfnxml('file'): index.File,
                 zfnxml('directory'): index.Directory,}

def _find_text(elem, tag):
    return _xml_strip(elem.findtext(zfnxml(tag)))

def _xml_strip(data):
    if data is None:
        return data
    data = dedent(data)
    if data.startswith('\n'):
        data = data[1:]
        if data.endswith('\n'):
            data = data[:-1]
    return data

def _decode_date(data):
    return datetime.strptime(data, "%Y-%m-%dT%H:%M:%SZ")

def decode_indexobj(elem, parent=None):
    """
    Decodes an object from an XML element.
    
    :Parameters:
        elem : xml.etree.ElementTree.Element
            The object's XML representation
    :Returns: The represented object
    :ReturnType: `zfilenet.indexserver.index.IndexObject`
    """
    # Determine correct tag
    try:
        factory = INDEXOBJ_TAGS[elem.tag]
    except KeyError:
        raise ValueError("Unrecognized tag: %s" % elem.tag)
    # Decode common data
    obj = factory(_find_text(elem, "name"), parent, elem.get("uuid"))
    # Decode specific data
    if elem.tag == zfnxml('file'):
        obj.length = int(_find_text(elem, "length"), 10)
        obj.sha1_hash = hex2data(_find_text(elem, "checksum"))
        blob_ids = [_xml_strip(child.text)
                    for child in elem.find(zfnxml("blob-list"))]
        obj.blob_list = blob_ids
    elif elem.tag == zfnxml('directory'):
        for child in elem:
            if child.tag in INDEXOBJ_TAGS:
                decode_indexobj(child, obj)
    # Decode dates (we do it here so we are not perturbed by other
    # modifications)
    obj.created = _decode_date(_find_text(elem, "created"))
    obj.modified = _decode_date(_find_text(elem, "last-modified"))
    obj.accessed = _decode_date(_find_text(elem, "last-accessed"))
    # Return result
    return obj

@progtools.reportexits
def main(args=None):
    # Get arguments
    if args is None:
        args = sys.argv[1:]
    if len(args) != 1:
        raise progtools.UsageError()
    # Connect to index
    if os.path.exists(args[0]):
        print >> sys.stderr, "%s: Database exists" % \
            (progtools.get_program_name())
        return 1
    index_db = index.connect(args[0])
    session = index.create_session(bind=index_db)
    input_file = progtools.open_input_file('-', 'rb')
    # Load document
    doc = etree.parse(input_file)
    input_file.close()
    # Handle header
    root = doc.getroot()
    if root.tag != zfnxml("zfn-index"):
        print >> sys.stderr, "%s: Unrecognized format" % \
            (progtools.get_program_name())
        return 2
    elif root.get('version') not in ('0.1', '0.2'):
        print >> sys.stderr, "%s: Unrecognized index version: %r" % \
            (progtools.get_program_name(), root.get('version'))
        return 2
    # Change root UUID
    index.get_root(session).delete()
    root_uuid = to_uuid(_find_text(root, "root-uuid"))
    DBVariable.set_variable(session, 'root_directory', root_uuid.hex)
    session.commit()
    # Parse top-level objects
    for child in root:
        if child.tag in INDEXOBJ_TAGS:
            obj = decode_indexobj(child)
            session.save(obj)
            session.commit()
    # Exit successfully
    session.close()
    return 0

if __name__ == '__main__':
    sys.exit(main())
