#!/usr/bin/env python
#
#   zfilenet/tests/test_blobs.py
#   zfilenet
#
#   Copyright (C) 2008 Ross Light
#
#   This file is part of zfilenet.
#
#   zfilenet is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   zfilenet is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with zfilenet.  If not, see <http://www.gnu.org/licenses/>.
#

"""Testing module for the blob server"""

from base64 import b64encode, b64decode
import unittest

from pinet.jsonrpc import RPCError
from zfilenet.blobserver import connect, create_session, DataBlob, BlobServer
from zfilenet.util import to_uuid

__author__ = 'Ross Light'
__date__ = 'July 18, 2008'
__all__ = ['DataTest',
           'ServerTest',]

_fake_uuid1 = "01234567-89ab-cdef-0123-456789abcdef"
_fake_uuid2 = "fedcba98-7654-3210-fedc-ba9876543210"
_sample_data = "hello\x00, world!\x00\x01\xff"
_big_data = "\x00" * (128 * 1024) # 128 KiB of nothing!

class DataTest(unittest.TestCase):
    def setUp(self):
        # Set up database
        self.engine = connect(':memory:')
        self.session = create_session(bind=self.engine)
        # Create initial test blob
        self.blob = DataBlob.write_blob(
            self.session, _fake_uuid1, _sample_data)
    
    def tearDown(self):
        self.session.close()
    
    def test_reading(self):
        """Blob read test"""
        fetch_blob = DataBlob.fetch_blob(self.session, _fake_uuid1)
        assert fetch_blob is self.blob, "Wrong blob returned"
        read_data = DataBlob.read_blob(self.session, _fake_uuid1)
        assert read_data == _sample_data, "Data corruption"
    
    def test_writing(self):
        """Blob write test"""
        assert self.blob.uuid == to_uuid(_fake_uuid1), "Wrong UUID"
        assert self.blob.data == _sample_data, "Wrong data"
        assert self.blob.size == len(_sample_data), "Wrong data size"
    
    def test_deleting(self):
        """Blob delete test"""
        fetch_blob = DataBlob.fetch_blob(self.session, _fake_uuid1)
        assert fetch_blob is not None, "Blob not entered"
        # Delete blob
        DataBlob.delete_blob(self.session, _fake_uuid1)
        fetch_blob = DataBlob.fetch_blob(self.session, _fake_uuid1)
        assert fetch_blob is None, "Blob not deleted"

class ServerTest(unittest.TestCase):
    quota = 128 * 1024
    
    def setUp(self):
        self.engine = connect(':memory:')
        self.server = BlobServer(self.engine, self.quota)
    
    def test_version(self):
        """Blob server should return module version"""
        from zfilenet import blobserver
        assert self.server.get_version() == blobserver.__version__
    
    def test_read(self):
        """Blob server read test"""
        assert self.server.read(_fake_uuid1) is None, "Bogus data generated"
        self.server.write(_fake_uuid1, b64encode(_sample_data))
        result = self.server.read(_fake_uuid1)
        assert result is not None, "No data returned"
        assert b64decode(result) == _sample_data, "Data corruption"
    
    def test_excessive_write(self):
        """Writing a big blob should raise an error"""
        try:
            self.server.write(_fake_uuid1, b64encode(_big_data + '\0'))
        except RPCError, e:
            assert e.code == -2, "Wrong error code for large blob"
        else:
            raise AssertionError("No error raised for large data")
    
    def test_overwriting(self):
        """Overwriting a blob should raise an error"""
        self.server.write(_fake_uuid1, b64encode(_sample_data))
        try:
            self.server.write(_fake_uuid1, b64encode(_sample_data))
        except RPCError, e:
            assert e.code == -1, "Wrong error code for overwrite"
        else:
            raise AssertionError("No error raised for overwrite")
    
    def test_over_quota(self):
        """Servers should honor quotas"""
        self.server.write(_fake_uuid1, b64encode(_big_data))
        try:
            self.server.write(_fake_uuid2, b64encode(_big_data))
        except RPCError, e:
            assert e.code == -3, "Wrong error code for over-quota"
        else:
            raise AssertionError("Over quota!")
    
    def test_limits(self):
        """Server should return correct limits"""
        result = self.server.get_limits()
        assert result == (128 * 1024, 0, self.quota), "Wrong limits given"
        self.server.write(_fake_uuid1, b64encode(_big_data))
        result = self.server.get_limits()
        assert result == (128 * 1024, len(_big_data), self.quota), \
            "Space used value not updated"
    
    def test_list(self):
        """Server listings should work properly"""
        # Test empty list
        result = self.server.get_blob_list()
        assert result == [], "Bogus blobs present"
        self.server.write(_fake_uuid1, b64encode(_sample_data))
        self.server.write(_fake_uuid2, b64encode(_sample_data))
        # Test all list
        result = [to_uuid(blob_id) for blob_id in self.server.get_blob_list()]
        assert len(result) == 2, "Wrong blob count"
        assert to_uuid(_fake_uuid1) in result, "Blob 1 not in list"
        assert to_uuid(_fake_uuid2) in result, "Blob 2 not in list"
        # Test sub-list
        result = self.server.get_blob_list([_fake_uuid1])
        result = [to_uuid(blob_id) for blob_id in result]
        assert len(result) == 1, "Wrong blob count"
        assert to_uuid(_fake_uuid1) in result, "Blob 1 not in sub-list"

if __name__ == '__main__':
    unittest.main()
